@@ -6,8 +6,8 @@
 ;; Maintainer: Pierre Rouleau <prouleau001@gmail.com>
 ;; URL: https://github.com/pierre-rouleau/tab-based-indent
 ;; Created   : Monday, November 10 2025.
-;; Package-Version: 20251125.1304
-;; Package-Revision: 5e83d2ff760c
+;; Package-Version: 20251201.2003
+;; Package-Revision: 473e3bd7e13f
 ;; Keywords: convenience, languages
 ;; Package-Requires: ((emacs "24.3"))
 
@@ -52,8 +52,8 @@
 ;; file.
 ;;
 ;; This feature may appeal to people that have problems working with small
-;; indentation width imposed by language or team conventions.  It acts as a
-;; workaround: temporary change the indentation scheme to a hard-tab based
+;; indentation width imposed by language or team code guidelines.  It acts as
+;; a workaround: temporary change the indentation scheme to a hard-tab based
 ;; indentation for editing, change the width of the hard tab for visibility
 ;; but keep the original indentation scheme inside the file.
 ;;
@@ -68,7 +68,7 @@
 ;;   and the width of the variables for the major mode to a larger value.
 ;;
 ;; Once set-up we can see the code with a wider indentation and
-;; continue to work with the rules imposed by the major mode logic.
+;; continue to edit the buffer with the rules imposed by the major mode logic.
 ;;
 ;; Later, before saving the buffer back to the file, we simply perform the
 ;; following steps:
@@ -90,14 +90,33 @@
 ;; ---------------------------------------------------------------------------
 ;;; History:
 ;;
-;; - Version 0.2.1 : Improve title.
-;; - Version 0.2   : Enhance safety: do not activate tbindent-mode if the
-;;                   indentation variable(s) for the major mode is unknown.
-;; - Version 0.1.01: Update description, docstring typo fix, allow Emacs 24.3.
-;; - Version 0.1: created, November 10, 2025 from code taken on my PEL project
-;;                making it self-sufficient allowing future publishing on
-;;                MELPA.  Restricted to Emacs 29.1 to comply with package-lint
-;;                despite the false positives reports.
+;; - Version 0.3.2 :
+;;     - Improved docstrings and comments.
+;; - Version 0.3.1 :
+;;     - Enhance safety: when `tab-width' differs from indentation control
+;;       variable, allow automatic adjustment of `tab-width' only when the
+;;       `dtrt-indent' mode is currently active;  we can trust the value of
+;;       the indentation control variable more when `dtrt-indent' has adjusted
+;;       it from the content of the file.
+;; - Version 0.3.0 :
+;;     - Simplify use: tbindent-mode automatically adjusts `tab-width' if it
+;;       differs from the value of the major mode indentation control
+;;       variable; it no longer requires the user to do it manually.  If the
+;;       indentation control variable is unknown for the major mode then it
+;;       issues an error with instructions.
+;;     - All messages start with "tbindent:"
+;; - Version 0.2.1:
+;;     Improve title.
+;; - Version 0.2.0:
+;;     Enhance safety: do not activate tbindent-mode if the indentation
+;;     variable(s) for the major mode is unknown.
+;; - Version 0.1.01:
+;;     Update description, docstring typo fix, allow Emacs 24.3.
+;; - Version 0.1:
+;;     Created, November 10, 2025 from code taken on my PEL project making it
+;;     self-sufficient allowing future publishing on MELPA.  Restricted to
+;;     Emacs 29.1 to comply with package-lint despite the false positives
+;;     reports.
 
 ;;; --------------------------------------------------------------------------
 ;;; Dependencies:
@@ -105,7 +124,7 @@
 ;;  Just Emacs provided packages:
 (require 'simple)         ; use: `normal-auto-fill-function'
 ;;
-;; The following variables are defined in Emacs built-ins:
+;; Use the following Emacs built-in variables:
 ;; from: Emacs files.el   ; use: `before-save-hook', `after-save-hook',
 ;;                        ;      `kill-buffer-hook'
 ;; from: Emacs indent.el  ; use: `standard-indent'
@@ -134,8 +153,9 @@
 
 (defcustom tbindent-target-indent-width-default 4
   "Default target indentation width.
-The indentation and tab width used by the `tbindent-mode' if none
-is specified in `tbindent-target-indent-widths' for the mode."
+The default indentation and tab width used by the `tbindent-mode'
+when there is no entry in `tbindent-target-indent-widths' for the current
+major mode."
   :group 'tbindent
   :type 'integer
   :safe 'tbindent-indent-valid-p)
@@ -157,28 +177,35 @@ mode."
 
 (defcustom tbindent-extra-mode-indent-vars nil
   "User-specified indentation variable specifications for modes.
-This is a alist mapping the major mode name to the name of one variable, a list
-of variables or a list of (vars . offset).  This identifies the name of the
-indentation control variable or variables used by the mode and if necessary an
-width offset applied to the variable: var = width + offset.
+This alist, used by `tbindent-mode' and `tbindent-set-tab-width',
+ maps the major mode name to one of:
 
-By adding entries into this list, you can add information that complements or
-overrides the entries in the hard-coded `tbindent--mode-indent-vars'.
+- the name of an indentation control variable used by the mode,
+- a list of the indentation variables used by the mode,
+- a list of (var . offset) cons cells where
 
-For example if you use the old `ada-mode' you could add the entry that maps it
-to `ada-indent' variable."
+  - the var is the name of one indentation control variable name, and
+  - offset, an integer, applied to requested width to compute the value
+    of the variable, like so: var = width + offset.
+
+By adding entries into this list, you can add information that
+complements or overrides entries in the hard-coded
+`tbindent--mode-indent-vars'.
+
+For example if you use the old `ada-mode' you could add the entry that
+maps it to `ada-indent' variable."
   :group 'tbindent
   :type '(repeat
           (list
-           (symbol :tag "mode name   ")
+           (symbol :tag "Major mode")
            (choice :tag "use"
-                   (symbol :tag "indent control variable name")
-                   (repeat :tag "Several variables"
-                    (symbol :tag "indent control variable name"))
-                   (repeat :tag "Several (var . offset) cells"
-                    (cons
-                     (symbol :tag "indent control variable name ")
-                     (integer :tag "offset from tab-width applied")))))))
+                   (symbol :tag "Indent control variable")
+                   (repeat :tag "Indent control variables"
+                           (symbol :tag "variable name"))
+                   (repeat :tag "(variable . offset) cells"
+                           (cons
+                            (symbol :tag "variable name")
+                            (integer :tag "applied offset")))))))
 
 ;;* Mode Specific Indentation Width Utilities
 ;;  -----------------------------------------
@@ -353,17 +380,16 @@ to `ada-indent' variable."
 This alist maps the mode name to one of 3 possible entities:
 - The name of the single variable that controls indentation for the mode, and
   which must have the same value as `tab-width'.
-- A list holding the names of each variable that control various aspects of
-  the mode's indentation.  Each of these variables must be set to the same
+- A list holding the names of each variable that control the mode's
+  indentation.  The mode sets Each of these variables to the same
   value as `tab-width'.
 - A list of (varname . offset) cons cell(s).  The car of the cons cell is the
   name of the indentation control variable.  The cdr of the cons cell is the
   offset that must be applied to `tab-width' to get the indentation value.
 
 IMPORTANT:
- Note that the `tbindent-mode' only works for buffer where the `tab-width'
- can be set to the same value as the indentation variable or all indentation
- variables.")
+ Note that `tbindent-mode' works reliably for major modes that use one or
+ mode indentation control variables without any offset.")
 
 
 (defun tbindent-string-ends-with-p (text suffix)
@@ -404,9 +430,10 @@ Return nil if none found."
 (defun tbindent-mode-indentation-width (&optional mode)
   "Return the indentation width used by current major mode or MODE.
 Return the value of the indentation control variable used for the
-current major mode (or the specified MODE) if there is one.  If there
-are several, return the value of the first one.  Return the value of
-`standard-indent' otherwise."
+current major mode (or the specified MODE) if there is one.  If more than one
+control variable is used, the first one should represent a full tab width
+and its value is returned.
+Return nil when it is unknown."
   (let ((vars (tbindent-mode-indent-control-vars mode)))
     (when vars
       (symbol-value (car vars)))))
@@ -490,11 +517,15 @@ If nothing exists for the current major-mode return DEFAULT-VALUE."
 ;;
 ;; * `tbindent-set-tab-width'
 ;;   - `tbindent-mode-indent-control-vars'
+;;     - `tbindent--indent-vars-for'
+;;       -d: tbindent-extra-mode-indent-vars
+;;       -d: tbindent--mode-indent-vars
+;;     - `tbindent-string-ends-with-p'
 ;;   - `tbindent-read-number'
 ;;
 
 (defvar-local tbindent--original-tab-width nil
-  "Tab width value used before `tbindent-indent-with-tabs' is used.")
+  "Tab width value used before `tbindent-indent-with-tabs' starts.")
 
 (defvar-local tbindent--last-set-tab-width nil
   "Tab width set by last `tbindent-set-tab-width' call.")
@@ -527,7 +558,9 @@ Return the new `tab-width' or nil if unchanged."
                                     'tbindent-set-tab-width-history)))
     ;;
     (when (not (= n current-tab-width))
-      (message "Changed buffer's tab-width from %d to %d" current-tab-width n)
+      (message
+       "tbindent: changed buffer's tab-width from %d to %d"
+       current-tab-width n)
       (when control-vars
         (dolist (var control-vars)
           (if (consp var)
@@ -826,6 +859,9 @@ Read the value from the `tbindent-target-indent-widths'.  If not found return
         (cdr spec)
       tbindent-target-indent-width-default)))
 
+(defvar-local tbindent--original-tab-width nil
+  "Value of buffer `tab-width' used before `tbindent-mode' activation.")
+
 ;;;###autoload
 (define-minor-mode tbindent-mode
   "Minor mode that automatically converts buffer to tab-based indentation.
@@ -833,105 +869,129 @@ Read the value from the `tbindent-target-indent-widths'.  If not found return
 Once the mode is active, change the visual indentation with the
 `tbindent-set-tab-width' command with \\[tbindent-set-tab-width].
 
-IMPORTANT: Note that `tbindent-mode' only works for buffer where
-`tab-width' has the same value as the indentation control variable.  It
-checks that when activating the mode and will refuse to activate it when
-the conditions are not met issuing a descriptive user-error instead."
+IMPORTANT: Note that `tbindent-mode' only works reliably for major modes
+that use one or mode indentation control variables without any offset
+and these all have the same value as `tab-width'.  It checks that when
+activating the mode and will refuse to activate it when the conditions
+are not met issuing a descriptive user-error instead."
   :lighter tbindent-lighter
   (let ((warning-message-printed nil)
         (mode-indentation-width (tbindent-mode-indentation-width)))
     (if tbindent-mode
         ;; When turning mode on
         ;; --------------------
-        (progn
-          (if (and mode-indentation-width
-                   (eq tab-width mode-indentation-width))
-              ;; Meeting all conditions to activate the mode
-              (progn
-                ;; On modified buffer, allow user to save first.  If user
-                ;; quit, catch and activate the mode anyway, without saving.
-                (condition-case nil
-                    (when (and (buffer-modified-p)
-                               (y-or-n-p (format "Save modified %S first? "
-                                                 (current-buffer))))
-                      (save-buffer))
-                  (quit
-                   (message
-                    "Indenting with tabs Mode enabled, buffer not saved!")
-                   (setq warning-message-printed t)))
-                ;; Proceed
-                (unless warning-message-printed
-                  (message "Converting %s to tab-based indent, width=%d ..."
-                           (current-buffer)
-                           tab-width ))
-                (with-silent-modifications
-                  ;; Remember the original space based indentation width
-                  (setq-local tbindent--space-based-indent-width
-                              (tbindent-mode-indentation-width))
+        (if (and mode-indentation-width
+                 (or (eq tab-width mode-indentation-width)
+                     (and (featurep 'dtrt-indent)
+                          (boundp 'dtrt-indent)
+                          dtrt-indent)))
+            ;; Indentation width variable is known, and either `tab-width'
+            ;; equals the indentation indent control variable value OR
+            ;; `dtrt-indent' mode is used and active (in which case the
+            ;; indentation control variable value can be trusted).
+            (progn
+              ;; Remember original value of `tab-width'.
+              (setq tbindent--original-tab-width tab-width)
+              ;; If available use dtrt-indent to identify indentation.
+              (when (fboundp 'dtrt-indent-mode)
+                (dtrt-indent-mode 1))
+              ;; Adjust local tab-width is necessary.
+              (unless (eq tab-width mode-indentation-width)
+                (message "tbindent: changing %s value of tab-width from %d to %d"
+                         (current-buffer)
+                         tab-width
+                         mode-indentation-width)
+                (setq-local tab-width mode-indentation-width))
+              ;; On modified buffer, allow user to save first.  If user
+              ;; quit, catch and activate the mode anyway, without saving.
+              (condition-case nil
+                  (when (and (buffer-modified-p)
+                             (y-or-n-p (format "Save modified %S first? "
+                                               (current-buffer))))
+                    (save-buffer))
+                (quit
+                 (message
+                  "tbindent on: indenting with tabs enabled, buffer not saved!")
+                 (setq warning-message-printed t)))
+              ;; Proceed
+              (unless warning-message-printed
+                (message
+                 "tbindent on: converting %s to tab-based indent, width=%d ..."
+                 (current-buffer)
+                 tab-width ))
+              (with-silent-modifications
+                ;; Remember the original space based indentation width
+                (setq-local tbindent--space-based-indent-width
+                            (tbindent-mode-indentation-width))
 
-                  ;; activate indentation with tabs using either the
-                  ;; indentation width specified by customization (if that
-                  ;; symbol exists and is non-nil or the native tab-width
-                  ;; matching indentation width
-                  (tbindent-indent-with-tabs
-                   (tbindent-target-indent-width-for major-mode)
-                   :by-minor-mode)
-                  ;; Install a special auto-fill function that is aware that
-                  ;; each tab in the buffer corresponds to the file original
-                  ;; space indentation scheme.
-                  (tbindent--install-indented-with-tabs-auto-fill))
-                ;; The buffer was modified by replacing spaces with tabs but
-                ;; since we want to use it as if it was normal, don't show
-                ;; the buffer modified unless it already was.
-                (unless warning-message-printed
-                  (set-buffer-modified-p nil))
-                ;; schedule operation before and after buffer save.
-                (unless (memq 'tbindent--before-save-or-kill  before-save-hook)
-                  (add-hook 'before-save-hook #'tbindent--before-save-or-kill
-                            -100
-                            'local))
-                (unless (memq 'tbindent--before-save-or-kill kill-buffer-hook)
-                  (add-hook 'kill-buffer-hook #'tbindent--before-save-or-kill
-                            -100
-                            'local))
-                (unless (memq 'tbindent--after-save after-save-hook)
-                  (add-hook 'after-save-hook #'tbindent--after-save
-                            +100
-                            'local))
+                ;; activate indentation with tabs using either the
+                ;; indentation width specified by customization (if that
+                ;; symbol exists and is non-nil or the native tab-width
+                ;; matching indentation width
+                (tbindent-indent-with-tabs
+                 (tbindent-target-indent-width-for major-mode)
+                 :by-minor-mode)
+                ;; Install a special auto-fill function that is aware that
+                ;; each tab in the buffer corresponds to the file original
+                ;; space indentation scheme.
+                (tbindent--install-indented-with-tabs-auto-fill))
+              ;; The buffer was modified by replacing spaces with tabs but
+              ;; since we want to use it as if it was normal, don't show
+              ;; the buffer modified unless it already was.
+              (unless warning-message-printed
+                (set-buffer-modified-p nil))
+              ;; schedule operation before and after buffer save.
+              (unless (memq 'tbindent--before-save-or-kill  before-save-hook)
+                (add-hook 'before-save-hook #'tbindent--before-save-or-kill
+                          -100
+                          'local))
+              (unless (memq 'tbindent--before-save-or-kill kill-buffer-hook)
+                (add-hook 'kill-buffer-hook #'tbindent--before-save-or-kill
+                          -100
+                          'local))
+              (unless (memq 'tbindent--after-save after-save-hook)
+                (add-hook 'after-save-hook #'tbindent--after-save
+                          +100
+                          'local))
 
-                (unless warning-message-printed
-                  (message "Indenting with tabs Mode enabled; width=%d."
-                           tab-width)))
-            ;; Not meeting all conditions!
-            (setq-local tbindent-mode nil)
-            (if mode-indentation-width
-                (user-error "\
-Can't activate tbindent-mode in %s: tab-width (%d) differs from %s (%d)!
-These must be the same and must represent the real indentation width used.
-To change tab-width, type:  M-: (setq-local tab-width %d)"
-                            (current-buffer)
-                            tab-width
-                            (tbindent-mode-indent-control-vars)
-                            mode-indentation-width
-                            mode-indentation-width)
+              (unless warning-message-printed
+                (message "tbindent on: indenting with tabs enabled; width=%d."
+                         tab-width)))
+          ;; Indentation control variable is unknown!
+          (setq-local tbindent-mode nil)
+          (if mode-indentation-width
               (user-error "\
-Can't activate tbindent-mode in %s: Unknown indentation control variable for '%s'!
-Please identify it in the `tbindent-extra-mode-indent-vars' alist."
+Can't activate tbindent-mode in %s: tab-width (%d) differs from %s (%d)!
+These must be the same.  Please use dtrt-indent.  See README.
+To change tab-width, type:  M-: (setq-local tab-width %d)"
                           (current-buffer)
-                          major-mode))))
+                          tab-width
+                          (tbindent-mode-indent-control-vars)
+                          mode-indentation-width
+                          mode-indentation-width)
+            (user-error "\
+Can't activate tbindent-mode in %s: Unknown indentation control variable for '%s'!
+Please identify it in the `tbindent-extra-mode-indent-vars' alist"
+                        (current-buffer)
+                        major-mode)))
 
       ;; When turning mode off
       ;; ---------------------
+      ;; Restore original space-based indentation scheme
       (with-silent-modifications
         (tbindent-indent-with-spaces nil :by-minor-mode))
+      ;; Restore original variable values
+      (setq  tab-width tbindent--original-tab-width)
       (tbindent--restore-original-fill-function)
+      ;; Remove the hooks
       (when (memq 'tbindent--before-save-or-kill before-save-hook)
         (remove-hook 'before-save-hook #'tbindent--before-save-or-kill 'local))
       (when (memq 'tbindent--before-save-or-kill kill-buffer-hook)
         (remove-hook 'kill-buffer-hook #'tbindent--before-save-or-kill 'local))
       (when (memq 'tbindent--after-save after-save-hook)
         (remove-hook 'after-save-hook #'tbindent--after-save 'local))
-      (message "Indenting with tabs Mode disabled."))))
+      ;; Inform user.
+      (message "tbindent off: restored file indentation scheme."))))
 
 ;;; --------------------------------------------------------------------------
 (provide 'tbindent)
